"""

THE USER MUST EDIT THIS FILE TO DEFINE THE MESH / RESOLUTION / FILE LOCATIONS
ETC

Setup base data for ANUGA run

"""
import glob, os, time, shutil
import hashlib, json
import numpy
from anuga.utilities import spatialInputUtil as su
import anuga
from anuga_parallel import myid, barrier, send, receive, numprocs
# We need the boundary tags info to correctly cache the mesh
#from setup_boundary_conditions import boundary_info
import user_functions


#@@@@@@@@@@@@@@@@@#
#START EDITING HERE
#@@@@@@@@@@@@@@@@@#


#####################################################################
#
# SCENARIO SETUP
#
#####################################################################

## Name for scenario
#
scenario='tsunami1_template'

## Total model runtime in seconds (starts at time=0.)
#
finaltime=0.05*3600.0

## Model time between output file timesteps in seconds
#
yieldstep=20.0

## Spatial Information
#
# Give UTM Zone as an integer (negative for Southern Hemisphere)
# or a proj4string defining the coordinate system (which is more flexible)
projection_information = 47

#####################################################################
#
# INPUT DATA DEFINING MESH
#
#####################################################################

## Bounding polygon with default mesh resolution (as a max triangle area in m^2)
#
# Filename MUST be a .shp file, and contain a set of lines making up the boundary polygon
# The lines each have an attribute giving the boundary tag, in the attribute table
# column named (the value of boundary_tags_attribute_name)
#
# The starting/ending points on each line MUST EXACTLY AGREE so the whole
# thing makes a polygon.
#
# Make it in GIS by first making the bounding polygon, then splitting it at
# the appropriate locations, then adding the boundary tag attribute.
#
#bounding_polygon_file='../anuga/polygons/bounding_polygon.csv'
bounding_polygon_and_tags_file='../anuga/boundaries/Patong_bounding_polygon/Patong_bounding_polygon.shp'
boundary_tags_attribute_name='bndryTag'

default_res=0.5*500**2  

## Interior regions with maximum triangle area resolutions (m^2)
#
# Filenames can be .shp or .csv
#
# If this is an empty list, we can instead use breaklines + ptAreas defined spatially
#

interior_regions_data = [['../anuga/polygons/inundation_area.csv', 75],
                         ['../anuga/polygons/aoi.csv', 200],
                         ['../anuga/polygons/aos.csv', 900],
                         ['../anuga/polygons/sw_coast_original.csv', 7000],
                         ['../anuga/polygons/building_main.csv', 20],
                         ['../anuga/polygons/building_main_small.csv', 20],
                         ['../anuga/polygons/building_main_south.csv', 19],
                         ['../anuga/polygons/building_saddle.csv', 20]]
## BreakLines.
#
# List of filenames (can be .shp and/or .csv)
#
#breakline_files=glob.glob('../../GIS/Mesh/BreakLines/*.shp')
breakline_files=[]

## Riverwalls
#
# Filenames MUST all be .csv with at least x,y,z of riverwall, and and optional
# header describing the riverwallPar
#
# riverwall_csv_files=glob.glob('../../GIS/Mesh/RiverWalls/*.csv')
riverwall_csv_files = []

# If breaklines/riverwalls intersect with < this distance between the intersection and an
# existing point, then the existing point is moved to the intersection. Otherwise
# the intersection point is added
breakLine_intersection_point_movement_threshold=10. 

# Get mesh resolutions = region_point_areas [alternative to interior_polygons + res]
#ptAreas='../../GIS/Mesh/RegionResolutions/RegionResolutions.shp'
ptAreas=None



######################################################################
#
# INITIAL CONDITIONS (minimal)
#
# We prefer to define data for initial / boundary conditions in the files
# setup_initial_conditions 
#         and 
# setup_boundary_conditions
#
# However some variables are needed globally, so they can go here
#
######################################################################

## Initial 'flat' water surface
# 
tide = 0.0


#####################################################################
#
# Numerics
#
#####################################################################

## ANUGA's flow algorithm
#
flow_algorithm="DE1"

## Local extrapolation and flux updating
#
# Only set this to True if you know what you are doing
#
use_local_extrapolation_and_flux_updating=False


#########################################################################
#
# OUTPUT INFORMATION
#
#########################################################################

## Output tif details
#
# pixle size (m) for output tifs
output_tif_cellsize = 250.
# Polygon for output tif [either a filename, or if None the bounding_polygon is used]
output_tif_bounding_polygon = None 


## Storage of max quantities
#
# Stage/depth/speed/speedxDepth maxima are stored separately to the SWW file
# Need to determine whether they are updated every time step or less frequently
# (less frequently might save computational time ?? )
# Can also choose to ignore an initial time period
max_quantity_update_frequency=1 # 1 = Every time step, 2 = every second timestep, etc
max_quantity_collection_start_time=0. # Set this to e.g. 3600.*2 to ignore the first 2 hours


## sww file-size options
#
# If store_vertices_uniquely=True, then SWW files are larger and vertices are
# stored uniquely for every triangle. Otherwise, vertex values are averaged
# before storage, leading to smaller file sizes but a less accurate
# representation of what the algorithm is doing
store_vertices_uniquely=False 

## Checkpointing
#
# Need to restructure to allow checkpointing
# checkpoint_time=30

#@@@@@@@@@@@@@@@@@#
#STOP EDITING HERE
#@@@@@@@@@@@@@@@@@#


###########################################################################
#
# BELOW HERE, THINGS ARE DESIGNED NOT TO BE CHANGED
#
###########################################################################

# We can either use interior regions, or breaklines
if not interior_regions_data==[]:
    assert ptAreas is None, 'Cannot define both ptAreas and non-empty interior regions'


#bounding_polygon=su.read_polygon(bounding_polygon_file)
bounding_polygon_and_tags =\
    user_functions.read_boundary_tags_line_shapefile(bounding_polygon_and_tags_file, boundary_tags_attribute_name)
bounding_polygon = bounding_polygon_and_tags[0]
boundary_tags = bounding_polygon_and_tags[1]

breaklines=su.readListOfBreakLines(breakline_files)
riverwalls, riverwall_par=su.readListOfRiverWalls(riverwall_csv_files) #{} # Dictionary

if ptAreas is not None: 
    region_point_areas=su.readregion_point_areas(ptAreas, convert_length_to_area=True)
else:
    region_point_areas = None
# Hack to override resolution
# region_point_areas=[ region_point_areas[i][0:2]+[150*150*0.5] for i in range(len(region_point_areas))]

# Redefine interior_regions to contain the polygon data + resolutions
interior_regions = [ [su.read_polygon(ir[0]), ir[1] ] for ir in interior_regions_data ]

# Directory where sww files are written
output_basedir='../anuga/outputs/'

# Directory where we store .txt versions of processed spatial inputs
spatial_txt_outputDir='SpatialTxt/' 

# Deal with intersections in the bounding polygon / breaklines / riverwalls
bounding_polygon, breaklines, riverwalls =\
    su.add_intersections_to_domain_features(bounding_polygon, breaklines, riverwalls,\
                  point_movement_threshold=breakLine_intersection_point_movement_threshold,
                  verbose=True)

# Here we make a unique ID based on the all the mesh geometry inputs
# This tells us if we need to regenerate partitions, or use old ones
mesh_id_hash=hashlib.md5(json.dumps(\
            [bounding_polygon, interior_regions, riverwalls, \
             breaklines, region_point_areas, default_res, boundary_tags]) ).hexdigest()

# Fix the output tif bounding polygon
if output_tif_bounding_polygon is None:
    output_tif_bounding_polygon = bounding_polygon
else:
    output_tif_bounding_polygon = su.read_polygon(output_tif_bounding_polygon)


# Make proj4string from projection information
#
if type(projection_information) is int:
    # projection_information describes a UTM zone
    # e.g. '+units=m +ellps=WGS84 +zone=47 +south=False +proj=utm '
    if(projection_information<0):
        proj4string="+proj=utm +zone="+str(abs(projection_information))+" +south +datum=WGS84 +units=m +no_defs"
    else:
        proj4string="+proj=utm +zone="+str(projection_information)+" +datum=WGS84 +units=m +no_defs"

else:
    msg = 'Invalid projection information --  must be a proj4string, or an integer'+\
          ' defining a UTM zone [positive for northern hemisphere, negative for southern hemisphere]'
    raise Exception, msg

# Set up directories etc 
if(myid==0):
    runtime = time.strftime('%Y%m%d_%H%M%S', time.localtime()) 
    for i in range(1,numprocs):
        send(runtime,i)
else:
    runtime=receive(0)
barrier()
        
output_dir = output_basedir+'RUN_'+str(runtime)+'_'+scenario
partition_basedir='PARTITIONS/'
partition_dir= partition_basedir+'Mesh_'+str(mesh_id_hash) 
meshname=output_dir+'/mesh.tsh'

def setup_directory_structure():
    """
        Make output directories, send important data to files
    """
    #if(myid==0):
        # Make the base directory if it does not exist
    try:
        os.mkdir(output_basedir)
    except:
        pass

    # Make the output directory
    try:
        os.mkdir(output_dir)
    except:
        pass
    
    # Make the partitions directory (2 steps as it is recursive)
    try:
        os.mkdir(partition_basedir)
    except:
        pass

    try:
        os.mkdir(partition_dir)
    except:
        pass

    # Make the spatialTxt directory
    try:
        os.mkdir(spatial_txt_outputDir)
    except:
        pass
    
    #################################################################################
    # Export breaklines to txt for QC
    def savLines(lines,filename='check.txt'):
        newA=numpy.array(lines)
        numpy.savetxt(filename, newA)
        return

    if(myid==0):
        if interior_regions is not []:
            for i in range(len(interior_regions)):
                savLines(interior_regions[i][0], filename=spatial_txt_outputDir+'interior_region_'+str(i)+'.txt')
        if breaklines is not {}:
            for i in range(len(breaklines)):
                savLines(breaklines.values()[i], filename=spatial_txt_outputDir+'breakline_'+str(i)+'.txt')
        if riverwalls is not {}:
            for i in range(len(riverwalls)):
                savLines(riverwalls.values()[i], filename=spatial_txt_outputDir+'riverwall_'+str(i)+'.txt')

        savLines(bounding_polygon,filename=spatial_txt_outputDir+'boundingPoly.txt')

        if region_point_areas is not None:
            savLines(region_point_areas, filename=spatial_txt_outputDir+'regionPtAreas.txt')

        # Copy all the txt representations of these files to the output directory
        shutil.copytree(spatial_txt_outputDir, output_dir+'/'+spatial_txt_outputDir)
    else:
        pass

    return
