"""

Home for various 'user-defined' functions.

The user can define any new functions they need in this script, and make
small modifications to the main scripts to use it. 

This is usually nicer than making detailed hacky changes to the main scripts

If you have a nice function, consider pushing it to the ANUGA source

"""

import anuga
from anuga_parallel import myid, numprocs, finalize, barrier


def print_velocity_statistics(domain, MaxQuantities):
    """
    Utility for printing velocity stats in the evolve loop
    """
    # Print velocity statistics
    for i in range(numprocs):
        if(myid==i):
            # Peak speed info
            xx = domain.quantities['xmomentum'].centroid_values
            yy = domain.quantities['ymomentum'].centroid_values
            dd = domain.quantities['stage'].centroid_values - domain.quantities['elevation'].centroid_values 
            dd = dd*(dd>1.0e-03)+1.0e-03*(dd<=1.0e-03)
            vv = 1/dd*( (xx)**2 + (yy)**2)**0.5
            vv = vv*(dd>1.0e-03)
            print '    Processor ', myid
            print '    @ Peak velocity is: ', vv.max(), vv.argmax()
            print '     &- MaxSpeedHistory: ', MaxQuantities.max_speed.max()
            print '     %- FUF: ', domain.flux_update_frequency.mean() 
        else:
            pass
        barrier()
    # Make a newline
    if(myid==0): print ''

############################################################

def read_boundary_tags_line_shapefile(shapefile_name, tag_attribute='bndryTag'):
    """
    Read in the boundary lines + tags from an appropriately structured line shapefile

    Return the bounding polygon + dict with tags + indices
    """

    ## Step 1: Read the data
    import ogr
    
    data_source=ogr.Open(shapefile_name)

    assert data_source.GetLayerCount()==1, ' Bounding polygon + boundary tags file can only have 1 layer'

    layer=data_source.GetLayer()

    boundary_line_and_tags=[]

    for feature in layer:
        line = feature.GetGeometryRef().GetPoints()
        line = [ list(l) for l in line]
        tag = feature.GetField(tag_attribute)
        boundary_line_and_tags.append([line, tag])


    ## Step 2: Convert to bounding polygon + boundary tags in ANUGA format
    boundary_segnum = len(boundary_line_and_tags)
    # Initial values
    bounding_polygon=boundary_line_and_tags[0][0]
    boundary_tags={boundary_line_and_tags[0][1]: range(len(bounding_polygon)-1) }
    # 
    #import pdb
    #pdb.set_trace()
    for i in range(boundary_segnum-1):
        # Find the line which starts at the end point of bounding_polygon
        for j in range(boundary_segnum):
            blj = boundary_line_and_tags[j][0]
            if blj[0]==bounding_polygon[-1]:
                # Append all but the first point of blj to bounding_polygon
                old_lbp=len(bounding_polygon)
                extra_polygon = [blj[k] for k in range(1, len(blj))]
                bounding_polygon = bounding_polygon + extra_polygon
                # If we are on the last segment, then drop the final point
                # (for consistency with ANUGA format)
                if i==(boundary_segnum-2):
                    bounding_polygon = [bounding_polygon[k] for k in range(len(bounding_polygon)-1)]
                    new_tags = range(old_lbp-1, len(bounding_polygon))
                else:
                    new_tags = range(old_lbp-1, len(bounding_polygon)-1)
                
                # Either the key already exists and we append edge indices, OR
                # we add the key and the edge indices
                key = boundary_line_and_tags[j][1]
                if boundary_tags.has_key(key):
                    boundary_tags[key] = boundary_tags[key]+ new_tags
                else:
                    boundary_tags[key] = new_tags

    return bounding_polygon, boundary_tags 
